@@ -288,6 +288,7 @@ template <typename R> struct BaseArch : ArchAPI<R>
     }
     virtual WireId getConflictingPipWire(PipId /*pip*/) const override { return WireId(); }
     virtual NetInfo *getConflictingPipNet(PipId pip) const override { return getBoundPipNet(pip); }
+    virtual bool isPipInverting(PipId /*pip*/) const override { return false; }
 
     // Group methods
     virtual GroupId getGroupByName(IdStringList /*name*/) const override { return GroupId(); };
@@ -442,8 +443,17 @@ template <typename R> struct BaseArch : ArchAPI<R>
         });
     }
 
+    // Routing methods
+    virtual void expandBoundingBox(BoundingBox &bb) const override
+    {
+        bb.x0 = std::max(bb.x0 - 1, 0);
+        bb.y0 = std::max(bb.y0 - 1, 0);
+        bb.x1 = std::min(bb.x1 + 1, this->getGridDimX());
+        bb.y1 = std::min(bb.y1 + 1, this->getGridDimY());
+    }
+
     // Flow methods
-    virtual void assignArchInfo() override{};
+    virtual void assignArchInfo() override {};
 
     // --------------------------------------------------------------
     // These structures are used to provide default implementations of bel/wire/pip binding. Arches might want to
