@@ -19,7 +19,6 @@
  */
 
 #ifndef NO_GUI
-#include <QApplication>
 #include "application.h"
 #include "mainwindow.h"
 #endif
@@ -29,8 +28,9 @@
 
 #include <boost/algorithm/string.hpp>
 #include <boost/algorithm/string/join.hpp>
-#include <boost/filesystem/convenience.hpp>
+#include <boost/filesystem/path.hpp>
 #include <boost/program_options.hpp>
+#include <cinttypes>
 #include <fstream>
 #include <iostream>
 #include <random>
@@ -61,7 +61,7 @@
 #include <unistd.h>
 #endif
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__NetBSD__)
 #include <sys/sysctl.h>
 #endif
 
@@ -90,10 +90,14 @@ std::string proc_self_dirname()
         buflen--;
     return std::string(path, buflen);
 }
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || defined(__NetBSD__)
 std::string proc_self_dirname()
 {
+#ifdef __NetBSD__
+    int mib[4] = {CTL_KERN, KERN_PROC_ARGS, getpid(), KERN_PROC_PATHNAME};
+#else
     int mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1};
+#endif
     size_t buflen;
     char *buffer;
     std::string path;
@@ -193,7 +197,7 @@ void init_share_dirname()
         npnr_share_dirname = proc_share_path;
         return;
     }
-    proc_share_path = proc_self_path + "..\\share\\";
+    proc_share_path = proc_self_path + "..\\share\\" + "nextpnr\\";
     if (check_file_exists(proc_share_path, true)) {
         npnr_share_dirname = proc_share_path;
         return;
@@ -339,7 +343,7 @@ po::options_description CommandHandler::getGeneralOptions()
     general.add_options()("json", po::value<std::string>(), "JSON design file to ingest");
     general.add_options()("write", po::value<std::string>(), "JSON design file to write");
     general.add_options()("top", po::value<std::string>(), "name of top module");
-    general.add_options()("seed", po::value<int>(), "seed value for random number generator");
+    general.add_options()("seed", po::value<uint64_t>(), "seed value for random number generator");
     general.add_options()("randomize-seed,r", "randomize seed value for random number generator");
 
     general.add_options()(
@@ -371,13 +375,15 @@ po::options_description CommandHandler::getGeneralOptions()
     general.add_options()("freq", po::value<double>(), "set target frequency for design in MHz");
     general.add_options()("timing-allow-fail", "allow timing to fail in design");
     general.add_options()("no-tmdriv", "disable timing-driven placement");
+    general.add_options()("sdc", po::value<std::string>(), "Generic timing constraints SDC file to load");
     general.add_options()("sdf", po::value<std::string>(), "SDF delay back-annotation file to write");
     general.add_options()("sdf-cvc", "enable tweaks for SDF file compatibility with the CVC simulator");
     general.add_options()("no-print-critical-path-source",
                           "disable printing of the line numbers associated with each net in the critical path");
 
     general.add_options()("placer-heap-alpha", po::value<float>(), "placer heap alpha value (float, default: 0.1)");
-    general.add_options()("placer-heap-beta", po::value<float>(), "placer heap beta value (float, default: 0.9)");
+    general.add_options()("placer-heap-beta", po::value<float>(),
+                          "placer heap maximum placement density (float, default: 0.9)");
     general.add_options()("placer-heap-critexp", po::value<int>(),
                           "placer heap criticality exponent (int, default: 2)");
     general.add_options()("placer-heap-timingweight", po::value<int>(), "placer heap timing weight (int, default: 10)");
@@ -385,6 +391,8 @@ po::options_description CommandHandler::getGeneralOptions()
                           "allow placer to attempt up to max(10000, total cells^2 / N) iterations to place a cell (int "
                           "N, default: 8, 0 for no timeout)");
 
+    general.add_options()("static-dump-density", "write density csv files during placer-static flow");
+
 #if !defined(NPNR_DISABLE_THREADS)
     general.add_options()("parallel-refine", "use new experimental parallelised engine for placement refinement");
 #endif
@@ -420,7 +428,7 @@ void script_terminate_handler()
 void CommandHandler::setupContext(Context *ctx)
 {
     if (ctx->settings.find(ctx->id("seed")) != ctx->settings.end())
-        ctx->rngstate = ctx->setting<uint64_t>("seed");
+        ctx->rngseed(ctx->setting<uint64_t>("seed"));
 
     if (vm.count("verbose")) {
         ctx->verbose = true;
@@ -440,7 +448,7 @@ void CommandHandler::setupContext(Context *ctx)
     }
 
     if (vm.count("seed")) {
-        ctx->rngseed(vm["seed"].as<int>());
+        ctx->rngseed(vm["seed"].as<uint64_t>());
     }
 
     if (vm.count("threads")) {
@@ -449,10 +457,10 @@ void CommandHandler::setupContext(Context *ctx)
 
     if (vm.count("randomize-seed")) {
         std::random_device randDev{};
-        std::uniform_int_distribution<int> distrib{1};
+        std::uniform_int_distribution<uint64_t> distrib{1};
         auto seed = distrib(randDev);
-        ctx->rngseed(seed);
-        log_info("Generated random seed: %d\n", seed);
+        ctx->rngstate = seed;
+        log_info("Generated random seed: %" PRIu64 "\n", seed);
     }
 
     if (vm.count("slack_redist_iter")) {
@@ -539,6 +547,9 @@ void CommandHandler::setupContext(Context *ctx)
     if (vm.count("router2-alt-weights"))
         ctx->settings[ctx->id("router2/alt-weights")] = true;
 
+    if (vm.count("static-dump-density"))
+        ctx->settings[ctx->id("static/dump_density")] = true;
+
     // Setting default values
     if (ctx->settings.find(ctx->id("target_freq")) == ctx->settings.end())
         ctx->settings[ctx->id("target_freq")] = std::to_string(12e6);
@@ -555,7 +566,7 @@ void CommandHandler::setupContext(Context *ctx)
 
     ctx->settings[ctx->id("arch.name")] = std::string(ctx->archId().c_str(ctx));
     ctx->settings[ctx->id("arch.type")] = std::string(ctx->archArgsToId(ctx->archArgs()).c_str(ctx));
-    ctx->settings[ctx->id("seed")] = ctx->rngstate;
+    ctx->settings[ctx->id("seed")] = Property(ctx->rngstate, 64);
 
     if (ctx->settings.find(ctx->id("placerHeap/alpha")) == ctx->settings.end())
         ctx->settings[ctx->id("placerHeap/alpha")] = std::to_string(0.1);
@@ -596,6 +607,13 @@ int CommandHandler::executeMain(std::unique_ptr<Context> ctx)
                 std::ifstream f(filename);
                 if (!parse_json(f, filename, w.getContext()))
                     log_error("Loading design failed.\n");
+
+                if (vm.count("sdc")) {
+                    std::string sdc_filename = vm["sdc"].as<std::string>();
+                    std::ifstream sdc_stream(sdc_filename);
+                    ctx->read_sdc(sdc_stream);
+                }
+
                 customAfterLoad(w.getContext());
                 w.notifyChangeContext();
                 w.updateActions();
@@ -604,6 +622,7 @@ int CommandHandler::executeMain(std::unique_ptr<Context> ctx)
         } catch (log_execution_error_exception) {
             // show error is handled by gui itself
         }
+
         w.show();
 
         return a.exec();
@@ -615,6 +634,12 @@ int CommandHandler::executeMain(std::unique_ptr<Context> ctx)
         if (!parse_json(f, filename, ctx.get()))
             log_error("Loading design failed.\n");
 
+        if (vm.count("sdc")) {
+            std::string sdc_filename = vm["sdc"].as<std::string>();
+            std::ifstream sdc_stream(sdc_filename);
+            ctx->read_sdc(sdc_stream);
+        }
+
         customAfterLoad(ctx.get());
     }
 
