@@ -638,9 +638,14 @@ class SAPlacer
                         Loc old_loc = ctx->getBelLocation(old_bel);
                         Loc bound_loc = ctx->getBelLocation(bound->bel);
                         Loc root_loc = ctx->getBelLocation(ctx->getClusterRootCell(bound->cluster)->bel);
-                        BelId new_root = ctx->getBelByLocation(Loc(old_loc.x + (root_loc.x - bound_loc.x),
-                                                                   old_loc.y + (root_loc.y - bound_loc.y),
-                                                                   old_loc.z + (root_loc.z - bound_loc.z)));
+                        Loc new_loc =
+                                Loc(old_loc.x + (root_loc.x - bound_loc.x), old_loc.y + (root_loc.y - bound_loc.y),
+                                    old_loc.z + (root_loc.z - bound_loc.z));
+                        if (new_loc.x < 0 || new_loc.x >= ctx->getGridDimX())
+                            goto swap_fail;
+                        if (new_loc.y < 0 || new_loc.y >= ctx->getGridDimY())
+                            goto swap_fail;
+                        BelId new_root = ctx->getBelByLocation(new_loc);
                         if (new_root == BelId())
                             goto swap_fail;
                         for (auto cluster_cell : cluster2cell.at(bound->cluster)) {
@@ -732,7 +737,6 @@ class SAPlacer
     {
         IdString targetType = cell->type;
         Loc curr_loc = ctx->getBelLocation(cell->bel);
-        int count = 0;
 
         int dx = diameter, dy = diameter;
         if (cell->region != nullptr && cell->region->constr_bels) {
@@ -772,7 +776,6 @@ class SAPlacer
                 continue;
             if (locked_bels.find(bel) != locked_bels.end())
                 continue;
-            count++;
             return bel;
         }
     }
