@@ -301,30 +301,43 @@ class HeAPPlacer
             ++iter;
         }
 
-        // Apply saved solution
+        // Remove any previous binding
         for (auto &sc : solution) {
             CellInfo *cell = std::get<0>(sc);
             if (cell->bel != BelId())
                 ctx->unbindBel(cell->bel);
         }
+        // Apply saved solution
         for (auto &sc : solution) {
             CellInfo *cell;
             BelId bel;
             PlaceStrength strength;
             std::tie(cell, bel, strength) = sc;
+            // Just skip unbound cells here, these errors are handled just after
+            if (bel == BelId())
+                continue;
             ctx->bindBel(bel, cell, strength);
         }
 
+        // Find and display all errors to help in finding the root cause of issues
+        unsigned num_errors = 0;
         for (auto &cell : ctx->cells) {
             if (cell.second->isPseudo())
                 continue;
-            if (cell.second->bel == BelId())
-                log_error("Found unbound cell %s\n", cell.first.c_str(ctx));
-            if (ctx->getBoundBelCell(cell.second->bel) != cell.second.get())
-                log_error("Found cell %s with mismatched binding\n", cell.first.c_str(ctx));
-            if (ctx->debug)
+            if (cell.second->bel == BelId()) {
+                log_nonfatal_error("Found unbound cell '%s' of type '%s'\n", cell.first.c_str(ctx),
+                                   cell.second->type.c_str(ctx));
+                num_errors++;
+            } else if (ctx->getBoundBelCell(cell.second->bel) != cell.second.get()) {
+                log_nonfatal_error("Found mismatched binding for '%s' or type '%s'\n", cell.first.c_str(ctx),
+                                   cell.second->type.c_str(ctx));
+                num_errors++;
+            } else if (ctx->debug)
                 log_info("AP soln: %s -> %s\n", cell.first.c_str(ctx), ctx->nameOfBel(cell.second->bel));
         }
+        if (num_errors > 0) {
+            log_error("Stopping the program after %u errors found\n", num_errors);
+        }
 
         bool any_bad_placements = false;
         for (auto bel : ctx->getBels()) {
@@ -849,9 +862,10 @@ class HeAPPlacer
         // the simple greedy largest-macro-first approach.
         std::priority_queue<std::pair<int, IdString>> remaining;
         for (auto cell : solve_cells) {
-            remaining.emplace(chain_size[cell->name], cell->name);
+            remaining.emplace(chain_size[cell->name] * cfg.get_cell_legalisation_weight(ctx, cell), cell->name);
         }
         int ripup_radius = 2;
+        int chain_ripup_radius = std::max(max_x, max_y); // only ripup chains as last resort
         int total_iters = 0;
         int total_iters_noreset = 0;
         while (!remaining.empty()) {
@@ -862,7 +876,8 @@ class HeAPPlacer
             // Was now placed, ignore
             if (ci->bel != BelId())
                 continue;
-            // log_info("   Legalising %s (%s) %d\n", top.second.c_str(ctx), ci->type.c_str(ctx), top.first);
+            if (ctx->debug)
+                log_info("   Legalising %s (%s) priority=%d\n", top.second.c_str(ctx), ci->type.c_str(ctx), top.first);
             FastBels::FastBelsData *fb;
             fast_bels.getBelsForCellType(ci->type, &fb);
             int radius = 0;
@@ -877,7 +892,7 @@ class HeAPPlacer
             total_iters_noreset++;
             if (total_iters > int(solve_cells.size())) {
                 total_iters = 0;
-                ripup_radius = std::max(std::max(max_x, max_y), ripup_radius * 2);
+                ripup_radius = std::min(std::max(max_x, max_y), ripup_radius * 2);
             }
 
             if (total_iters_noreset > std::max(5000, 8 * int(ctx->cells.size()))) {
@@ -886,10 +901,11 @@ class HeAPPlacer
 
             while (!placed) {
                 if (cfg.cell_placement_timeout > 0 && total_iters_for_cell > cfg.cell_placement_timeout)
-                    log_error("Unable to find legal placement for cell '%s' after %d attempts, check constraints and "
+                    log_error("Unable to find legal placement for cell '%s' of type '%s' after %d attempts, check "
+                              "constraints and "
                               "utilisation. Use `--placer-heap-cell-placement-timeout` to change the number of "
                               "attempts.\n",
-                              ctx->nameOf(ci), total_iters_for_cell);
+                              ctx->nameOf(ci), ci->type.c_str(ctx), total_iters_for_cell);
 
                 // Determine a search radius around the solver location (which increases over time) that is clamped to
                 // the region constraint for the cell (if applicable)
@@ -959,7 +975,8 @@ class HeAPPlacer
                     CellInfo *bound = ctx->getBoundBelCell(bestBel);
                     if (bound != nullptr) {
                         ctx->unbindBel(bound->bel);
-                        remaining.emplace(chain_size[bound->name], bound->name);
+                        remaining.emplace(chain_size[bound->name] * cfg.get_cell_legalisation_weight(ctx, bound),
+                                          bound->name);
                     }
                     ctx->bindBel(bestBel, ci, STRENGTH_WEAK);
                     placed = true;
@@ -981,7 +998,7 @@ class HeAPPlacer
                             CellInfo *bound = ctx->getBoundBelCell(sz);
                             if (bound != nullptr) {
                                 // Only rip up cells without constraints
-                                if (bound->cluster != ClusterId())
+                                if (bound->cluster != ClusterId() || bound->belStrength > STRENGTH_WEAK)
                                     continue;
                                 ctx->unbindBel(bound->bel);
                             }
@@ -1021,7 +1038,9 @@ class HeAPPlacer
                             } else {
                                 // It's legal, and we've tried enough. Finish.
                                 if (bound != nullptr)
-                                    remaining.emplace(chain_size[bound->name], bound->name);
+                                    remaining.emplace(chain_size[bound->name] *
+                                                              cfg.get_cell_legalisation_weight(ctx, bound),
+                                                      bound->name);
                                 Loc loc = ctx->getBelLocation(sz);
                                 cell_locs[ci->name].x = loc.x;
                                 cell_locs[ci->name].y = loc.y;
@@ -1036,7 +1055,7 @@ class HeAPPlacer
                         // List of cells and their destination
                         std::vector<std::pair<CellInfo *, BelId>> targets;
                         // List of bels we placed things at; and the cell that was there before if applicable
-                        std::vector<std::pair<BelId, CellInfo *>> swaps_made;
+                        dict<BelId, CellInfo *> moves_made;
 
                         if (!ctx->getClusterPlacement(ci->cluster, sz, targets))
                             continue;
@@ -1047,31 +1066,45 @@ class HeAPPlacer
                                 goto fail;
                             CellInfo *bound = ctx->getBoundBelCell(target.second);
                             // Chains cannot overlap; so if we have to ripup a cell make sure it isn't part of a chain
-                            if (bound != nullptr)
-                                if (bound->cluster != ClusterId() || bound->belStrength > STRENGTH_WEAK)
+                            if (bound != nullptr) {
+                                if (bound->belStrength > (cfg.chainRipup ? STRENGTH_STRONG : STRENGTH_WEAK))
                                     goto fail;
+                                if (bound->cluster != ClusterId() && (!cfg.chainRipup || radius < chain_ripup_radius))
+                                    goto fail;
+                            }
                         }
                         // Actually perform the move; keeping track of the moves we make so we can revert them if needed
                         for (auto &target : targets) {
                             CellInfo *bound = ctx->getBoundBelCell(target.second);
-                            if (bound != nullptr)
-                                ctx->unbindBel(target.second);
+                            if (bound != nullptr) {
+                                if (bound->cluster != ClusterId()) {
+                                    for (auto cell : cluster2cells[bound->cluster]) {
+                                        if (cell->bel != BelId()) {
+                                            moves_made[cell->bel] = cell;
+                                            ctx->unbindBel(cell->bel);
+                                        }
+                                    }
+                                } else {
+                                    ctx->unbindBel(target.second);
+                                }
+                            }
                             ctx->bindBel(target.second, target.first, STRENGTH_STRONG);
-                            swaps_made.emplace_back(target.second, bound);
+                            moves_made[target.second] = bound;
                         }
                         // Check that the move we have made is legal
-                        for (auto &sm : swaps_made) {
-                            if (!ctx->isBelLocationValid(sm.first))
+                        for (auto &move : moves_made) {
+                            if (!ctx->isBelLocationValid(move.first))
                                 goto fail;
                         }
 
                         if (false) {
                         fail:
                             // If the move turned out to be illegal; revert all the moves we made
-                            for (auto &swap : swaps_made) {
-                                ctx->unbindBel(swap.first);
-                                if (swap.second != nullptr)
-                                    ctx->bindBel(swap.first, swap.second, STRENGTH_WEAK);
+                            for (auto &move : moves_made) {
+                                if (ctx->getBoundBelCell(move.first))
+                                    ctx->unbindBel(move.first);
+                                if (move.second != nullptr)
+                                    ctx->bindBel(move.first, move.second, STRENGTH_WEAK);
                             }
                             continue;
                         }
@@ -1081,10 +1114,12 @@ class HeAPPlacer
                             cell_locs[target.first->name].y = loc.y;
                             // log_info("%s %d %d %d\n", target.first->name.c_str(ctx), loc.x, loc.y, loc.z);
                         }
-                        for (auto &swap : swaps_made) {
+                        for (auto &move : moves_made) {
                             // Where we have ripped up cells; add them to the queue
-                            if (swap.second != nullptr)
-                                remaining.emplace(chain_size[swap.second->name], swap.second->name);
+                            if (move.second != nullptr && (move.second->cluster == ClusterId() || ctx->getClusterRootCell(move.second->cluster) == move.second))
+                                remaining.emplace(chain_size[move.second->name] *
+                                                          cfg.get_cell_legalisation_weight(ctx, move.second),
+                                                  move.second->name);
                         }
 
                         placed = true;
@@ -1543,12 +1578,10 @@ class HeAPPlacer
         {
             cut_cells.clear();
             auto &cal = cells_at_location;
-            int total_cells = 0, total_bels = 0;
+            int total_cells = 0;
             for (int x = r.x0; x <= r.x1; x++) {
                 for (int y = r.y0; y <= r.y1; y++) {
                     std::copy(cal.at(x).at(y).begin(), cal.at(x).at(y).end(), std::back_inserter(cut_cells));
-                    for (size_t t = 0; t < buckets.size(); t++)
-                        total_bels += bels_at(x, y, t);
                 }
             }
             for (auto &cell : cut_cells) {
@@ -1823,6 +1856,7 @@ PlacerHeapCfg::PlacerHeapCfg(Context *ctx)
     timing_driven = ctx->setting<bool>("timing_driven");
     solverTolerance = 1e-5;
     placeAllAtOnce = false;
+    chainRipup = false;
 
     int timeout_divisor = ctx->setting<int>("placerHeap/cellPlacementTimeout", 8);
     if (timeout_divisor > 0) {
