@@ -18,6 +18,7 @@
  */
 
 #include "arch.h"
+#include <boost/filesystem/path.hpp>
 #include "archdefs.h"
 #include "chipdb.h"
 #include "log.h"
@@ -32,7 +33,7 @@
 
 NEXTPNR_NAMESPACE_BEGIN
 
-static constexpr int database_version = 3;
+static constexpr int database_version = 6;
 
 static const ChipInfoPOD *get_chip_info(const RelPtr<ChipInfoPOD> *ptr) { return ptr->get(); }
 
@@ -62,8 +63,10 @@ void Arch::load_chipdb(const std::string &path)
         db_path = args.chipdb_override;
     } else {
         db_path = proc_share_dirname();
-        db_path += "/himbaechel/";
+        db_path += "himbaechel/";
         db_path += path;
+        boost::filesystem::path p(db_path);
+        db_path = p.make_preferred().string();
     }
     try {
         blob_file.open(db_path);
@@ -270,7 +273,7 @@ WireId Arch::getWireByName(IdStringList name) const
     const auto &tdata = chip_tile_info(chip_info, tile);
     for (int wire = 0; wire < tdata.wires.ssize(); wire++) {
         if (IdString(tdata.wires[wire].name) == name[1])
-            return WireId(tile, wire);
+            return normalise_wire(tile, wire);
     }
     return WireId();
 }
@@ -282,28 +285,53 @@ IdStringList Arch::getWireName(WireId wire) const
 
 PipId Arch::getPipByName(IdStringList name) const
 {
-    NPNR_ASSERT(name.size() == 3);
-    int tile = tile_name2idx.at(name[0]);
+    NPNR_ASSERT(name.size() == 3 || (name.size() == 4 && name[3] == id("INV")));
+    const int tile = tile_name2idx.at(name[0]);
     const auto &tdata = chip_tile_info(chip_info, tile);
     for (int pip = 0; pip < tdata.pips.ssize(); pip++) {
         if (IdString(tdata.wires[tdata.pips[pip].dst_wire].name) == name[1] &&
-            IdString(tdata.wires[tdata.pips[pip].src_wire].name) == name[2])
-            return PipId(tile, pip);
+            IdString(tdata.wires[tdata.pips[pip].src_wire].name) == name[2]) {
+
+            const auto tmp_pip = PipId(tile, pip);
+            if ((name.size() == 3 && !isPipInverting(tmp_pip)) || (name.size() == 4 && isPipInverting(tmp_pip))) {
+                return tmp_pip;
+            }
+        }
     }
     return PipId();
 }
 
 IdStringList Arch::getPipName(PipId pip) const
 {
-    auto &tdata = chip_tile_info(chip_info, pip.tile);
-    auto &pdata = tdata.pips[pip.index];
-    return IdStringList::concat(tile_name.at(pip.tile),
-                                IdStringList::concat(IdString(tdata.wires[pdata.dst_wire].name),
-                                                     IdString(tdata.wires[pdata.src_wire].name)));
+    const auto &tdata = chip_tile_info(chip_info, pip.tile);
+    const auto &pdata = tdata.pips[pip.index];
+    const auto name = IdStringList::concat(tile_name.at(pip.tile),
+                                           IdStringList::concat(IdString(tdata.wires[pdata.dst_wire].name),
+                                                                IdString(tdata.wires[pdata.src_wire].name)));
+    if (isPipInverting(pip))
+        return IdStringList::concat(name, id("INV"));
+    return name;
 }
 
 IdString Arch::getPipType(PipId pip) const { return IdString(); }
 
+GroupId Arch::getGroupByName(IdStringList name) const
+{
+    NPNR_ASSERT(name.size() == 2);
+    int tile = tile_name2idx.at(name[0]);
+    const auto &tdata = chip_tile_info(chip_info, tile);
+    for (int group = 0; group < tdata.groups.ssize(); group++) {
+        if (IdString(tdata.groups[group].name) == name[1])
+            return GroupId(tile, group);
+    }
+    return GroupId();
+}
+
+IdStringList Arch::getGroupName(GroupId group) const
+{
+    return IdStringList::concat(tile_name.at(group.tile), IdString(chip_group_info(chip_info, group).name));
+}
+
 std::string Arch::getChipName() const { return chip_info->name.get(); }
 
 IdString Arch::archArgsToId(ArchArgs args) const
@@ -384,13 +412,12 @@ bool Arch::lookup_cell_delay(int type_idx, IdString from_port, IdString to_port,
 {
     NPNR_ASSERT(type_idx != -1);
     const auto &ct = speed_grade->cell_types[type_idx];
-    int to_pin_idx = db_binary_search(
-            ct.pins, [](const CellPinTimingPOD &pd) { return pd.pin; }, to_port.index);
+    int to_pin_idx = db_binary_search(ct.pins, [](const CellPinTimingPOD &pd) { return pd.pin; }, to_port.index);
     if (to_pin_idx == -1)
         return false;
     const auto &tp = ct.pins[to_pin_idx];
-    int arc_idx = db_binary_search(
-            tp.comb_arcs, [](const CellPinCombArcPOD &arc) { return arc.input; }, from_port.index);
+    int arc_idx =
+            db_binary_search(tp.comb_arcs, [](const CellPinCombArcPOD &arc) { return arc.input; }, from_port.index);
     if (arc_idx == -1)
         return false;
     delay = DelayQuad(tp.comb_arcs[arc_idx].delay.fast_min, tp.comb_arcs[arc_idx].delay.slow_max);
@@ -401,8 +428,7 @@ const RelSlice<CellPinRegArcPOD> *Arch::lookup_cell_seq_timings(int type_idx, Id
 {
     NPNR_ASSERT(type_idx != -1);
     const auto &ct = speed_grade->cell_types[type_idx];
-    int pin_idx = db_binary_search(
-            ct.pins, [](const CellPinTimingPOD &pd) { return pd.pin; }, port.index);
+    int pin_idx = db_binary_search(ct.pins, [](const CellPinTimingPOD &pd) { return pd.pin; }, port.index);
     if (pin_idx == -1)
         return nullptr;
     return &ct.pins[pin_idx].reg_arcs;
@@ -413,8 +439,7 @@ TimingPortClass Arch::lookup_port_tmg_type(int type_idx, IdString port, PortType
 
     NPNR_ASSERT(type_idx != -1);
     const auto &ct = speed_grade->cell_types[type_idx];
-    int pin_idx = db_binary_search(
-            ct.pins, [](const CellPinTimingPOD &pd) { return pd.pin; }, port.index);
+    int pin_idx = db_binary_search(ct.pins, [](const CellPinTimingPOD &pd) { return pd.pin; }, port.index);
     if (pin_idx == -1)
         return (dir == PORT_OUT) ? TMG_IGNORE : TMG_COMB_INPUT;
     auto &pin = ct.pins[pin_idx];
@@ -429,14 +454,13 @@ TimingPortClass Arch::lookup_port_tmg_type(int type_idx, IdString port, PortType
     }
 }
 
-// TODO: adding uarch overrides for these?
-bool Arch::getCellDelay(const CellInfo *cell, IdString fromPort, IdString toPort, DelayQuad &delay) const
+bool Arch::get_cell_delay_default(const CellInfo *cell, IdString fromPort, IdString toPort, DelayQuad &delay) const
 {
     if (cell->timing_index == -1)
         return false;
     return lookup_cell_delay(cell->timing_index, fromPort, toPort, delay);
 }
-TimingPortClass Arch::getPortTimingClass(const CellInfo *cell, IdString port, int &clockInfoCount) const
+TimingPortClass Arch::get_port_timing_class_default(const CellInfo *cell, IdString port, int &clockInfoCount) const
 {
     if (cell->timing_index == -1)
         return TMG_IGNORE;
@@ -449,7 +473,7 @@ TimingPortClass Arch::getPortTimingClass(const CellInfo *cell, IdString port, in
     }
     return type;
 }
-TimingClockingInfo Arch::getPortClockingInfo(const CellInfo *cell, IdString port, int index) const
+TimingClockingInfo Arch::get_port_clocking_info_default(const CellInfo *cell, IdString port, int index) const
 {
     TimingClockingInfo result;
     NPNR_ASSERT(cell->timing_index != -1);
@@ -501,4 +525,73 @@ IdString Arch::get_tile_type(int tile) const
     return IdString(tile_data.type_name);
 }
 
+std::vector<GraphicElement> Arch::getDecalGraphics(DecalId decal) const
+{
+    std::vector<GraphicElement> ret;
+    if (decal.type == DecalId::TYPE_GROUP) {
+        GroupId group(decal.tile, decal.index);
+        Loc loc;
+        tile_xy(chip_info, decal.tile, loc.x, loc.y);
+        uarch->drawGroup(ret, group, loc);
+    } else if (decal.type == DecalId::TYPE_BEL) {
+        BelId bel(decal.tile, decal.index);
+        GraphicElement::style_t style = decal.active ? GraphicElement::STYLE_ACTIVE : GraphicElement::STYLE_INACTIVE;
+        uarch->drawBel(ret, style, getBelType(bel), getBelLocation(bel));
+    } else if (decal.type == DecalId::TYPE_WIRE) {
+        WireId w(decal.tile, decal.index);
+        for (WireId wire : get_tile_wire_range(w)) {
+            auto wire_type = getWireType(wire);
+            GraphicElement::style_t style =
+                    decal.active ? GraphicElement::STYLE_ACTIVE : GraphicElement::STYLE_INACTIVE;
+            Loc loc;
+            tile_xy(chip_info, wire.tile, loc.x, loc.y);
+            int32_t tilewire = chip_wire_info(chip_info, wire).tile_wire;
+            uarch->drawWire(ret, style, loc, wire_type, tilewire, get_tile_type(wire.tile));
+        }
+    } else if (decal.type == DecalId::TYPE_PIP) {
+        PipId pip(decal.tile, decal.index);
+        WireId src_wire = getPipSrcWire(pip);
+        WireId dst_wire = getPipDstWire(pip);
+        Loc loc = getPipLocation(pip);
+        int32_t src_id = chip_wire_info(chip_info, src_wire).tile_wire;
+        int32_t dst_id = chip_wire_info(chip_info, dst_wire).tile_wire;
+        GraphicElement::style_t style = decal.active ? GraphicElement::STYLE_ACTIVE : GraphicElement::STYLE_HIDDEN;
+        uarch->drawPip(ret, style, loc, src_wire, getWireType(src_wire), src_id, dst_wire, getWireType(dst_wire),
+                       dst_id);
+    }
+    return ret;
+}
+
+DecalXY Arch::getBelDecal(BelId bel) const
+{
+    DecalXY decalxy;
+    decalxy.decal = DecalId(bel.tile, bel.index, DecalId::TYPE_BEL);
+    decalxy.decal.active = getBoundBelCell(bel) != nullptr;
+    return decalxy;
+}
+
+DecalXY Arch::getWireDecal(WireId wire) const
+{
+    DecalXY decalxy;
+    decalxy.decal = DecalId(wire.tile, wire.index, DecalId::TYPE_WIRE);
+    decalxy.decal.active = getBoundWireNet(wire) != nullptr;
+    return decalxy;
+}
+
+DecalXY Arch::getPipDecal(PipId pip) const
+{
+    DecalXY decalxy;
+    decalxy.decal = DecalId(pip.tile, pip.index, DecalId::TYPE_PIP);
+    decalxy.decal.active = getBoundPipNet(pip) != nullptr;
+    return decalxy;
+}
+
+DecalXY Arch::getGroupDecal(GroupId group) const
+{
+    DecalXY decalxy;
+    decalxy.decal = DecalId(group.tile, group.index, DecalId::TYPE_GROUP);
+    decalxy.decal.active = true;
+    return decalxy;
+}
+
 NEXTPNR_NAMESPACE_END
