@@ -144,6 +144,7 @@ class TileWireData:
     index: int
     name: IdString
     wire_type: IdString
+    gfx_wire_id: int
     const_value: IdString = field(default_factory=list)
     flags: int = 0
     timing_idx: int = -1
@@ -166,6 +167,7 @@ class TileWireData:
     def serialise(self, context: str, bba: BBAWriter):
         bba.u32(self.name.index)
         bba.u32(self.wire_type.index)
+        bba.u32(self.gfx_wire_id)
         bba.u32(self.const_value.index)
         bba.u32(self.flags)
         bba.u32(self.timing_idx)
@@ -200,15 +202,59 @@ class PipData(BBAStruct):
         else:
             bba.u32(0)
 @dataclass
+class GroupData(BBAStruct):
+    index: int
+    name: IdString
+    group_type: IdString = field(default_factory=IdString)
+    group_bels: list[int] = field(default_factory=list)
+    group_wires: list[int] = field(default_factory=list)
+    group_pips: list[int] = field(default_factory=list)
+    group_groups: list[int] = field(default_factory=list)
+    extra_data: object = None
+
+    def serialise_lists(self, context: str, bba: BBAWriter):
+        bba.label(f"{context}_group_bels")
+        for idx in self.group_bels:
+            bba.u32(idx)
+        bba.label(f"{context}_group_wires")
+        for idx in self.group_wires:
+            bba.u32(idx)
+        bba.label(f"{context}_group_pips")
+        for idx in self.group_pips:
+            bba.u32(idx)
+        bba.label(f"{context}_group_groups")
+        for idx in self.group_groups:
+            bba.u32(idx)
+        # extra data (optional)
+        if self.extra_data is not None:
+            self.extra_data.serialise_lists(f"{context}_extra_data", bba)
+            bba.label(f"{context}_extra_data")
+            self.extra_data.serialise(f"{context}_extra_data", bba)
+    def serialise(self, context: str, bba: BBAWriter):
+        bba.u32(self.name.index)
+        bba.u32(self.group_type.index)
+        bba.slice(f"{context}_group_bels", len(self.group_bels))
+        bba.slice(f"{context}_group_wires", len(self.group_wires))
+        bba.slice(f"{context}_group_pips", len(self.group_pips))
+        bba.slice(f"{context}_group_groups", len(self.group_groups))
+        if self.extra_data is not None:
+            bba.ref(f"{context}_extra_data")
+        else:
+            bba.u32(0)
+
+@dataclass
 class TileType(BBAStruct):
     strs: StringPool
+    gfx_wire_ids: dict()
     tmg: "TimingPool"
     type_name: IdString
     bels: list[BelData] = field(default_factory=list)
     pips: list[PipData] = field(default_factory=list)
     wires: list[TileWireData] = field(default_factory=list)
+    groups: list[GroupData] = field(default_factory=list)
 
     _wire2idx: dict[IdString, int] = field(default_factory=dict)
+    _group2idx: dict[IdString, int] = field(default_factory=dict)
 
     extra_data: object = None
 
@@ -229,9 +275,13 @@ class TileType(BBAStruct):
 
     def create_wire(self, name: str, type: str="", const_value: str=""):
         # Create a new tile wire of a given name and type (optional) in the tile type
+        gfx_wire_id = 0
+        if name in self.gfx_wire_ids:
+            gfx_wire_id = self.gfx_wire_ids[name]
         wire = TileWireData(index=len(self.wires),
             name=self.strs.id(name),
             wire_type=self.strs.id(type),
+            gfx_wire_id=gfx_wire_id,
             const_value=self.strs.id(const_value))
         self._wire2idx[wire.name] = wire.index
         self.wires.append(wire)
@@ -246,6 +296,26 @@ class TileType(BBAStruct):
         self.wires[dst_idx].pips_uphill.append(pip.index)
         self.pips.append(pip)
         return pip
+    def create_group(self, name: str, type: str):
+        # Create a new group of a given name and type in the tile type
+        group = GroupData(index=len(self.groups),
+            name=self.strs.id(name),
+            group_type=self.strs.id(type))
+        self._group2idx[group.name] = group.index
+        self.groups.append(group)
+        return group
+    def add_bel_to_group(self, bel: BelData, group: str):
+        group_idx = self._group2idx[self.strs.id(group)]
+        self.groups[group_idx].group_bels.append(bel.index)
+    def add_wire_to_group(self, wire: TileWireData, group: str):
+        group_idx = self._group2idx[self.strs.id(group)]
+        self.groups[group_idx].group_wires.append(wire.index)
+    def add_pip_to_group(self, pip: PipData, group: str):
+        group_idx = self._group2idx[self.strs.id(group)]
+        self.groups[group_idx].group_pips.append(pip.index)
+    def add_group_to_group(self, sub_group: GroupData, group: str):
+        group_idx = self._group2idx[self.strs.id(group)]
+        self.groups[group_idx].group_groups.append(sub_group.index)
     def has_wire(self, wire: str):
         # Check if a wire has already been created
         return self.strs.id(wire) in self._wire2idx
@@ -260,6 +330,8 @@ class TileType(BBAStruct):
             wire.serialise_lists(f"{context}_wire{i}", bba)
         for i, pip in enumerate(self.pips):
             pip.serialise_lists(f"{context}_pip{i}", bba)
+        for i, group in enumerate(self.groups):
+            group.serialise_lists(f"{context}_group{i}", bba)
         # lists of members
         bba.label(f"{context}_bels")
         for i, bel in enumerate(self.bels):
@@ -270,6 +342,9 @@ class TileType(BBAStruct):
         bba.label(f"{context}_pips")
         for i, pip in enumerate(self.pips):
             pip.serialise(f"{context}_pip{i}", bba)
+        bba.label(f"{context}_groups")
+        for i, group in enumerate(self.groups):
+            group.serialise(f"{context}_group{i}", bba)
         # extra data (optional)
         if self.extra_data is not None:
             self.extra_data.serialise_lists(f"{context}_extra_data", bba)
@@ -280,6 +355,7 @@ class TileType(BBAStruct):
         bba.slice(f"{context}_bels", len(self.bels))
         bba.slice(f"{context}_wires", len(self.wires))
         bba.slice(f"{context}_pips", len(self.pips))
+        bba.slice(f"{context}_groups", len(self.groups))
         if self.extra_data is not None:
             bba.ref(f"{context}_extra_data")
         else:
@@ -411,6 +487,7 @@ class PackageInfo(BBAStruct):
     strs: StringPool
     name: IdString
     pads: list[int] = field(default_factory=list)
+    extra_data: object = None
 
     def create_pad(self, package_pin: str, tile: str, bel: str, pad_function: str, pad_bank: int, flags: int = 0):
         pad = PadInfo(package_pin = self.strs.id(package_pin), tile = self.strs.id(tile), bel = self.strs.id(bel),
@@ -424,10 +501,18 @@ class PackageInfo(BBAStruct):
         bba.label(f"{context}_pads")
         for i, pad in enumerate(self.pads):
             pad.serialise(f"{context}_pad{i}", bba)
+        if self.extra_data is not None:
+            self.extra_data.serialise_lists(f"{context}_extra_data", bba)
+            bba.label(f"{context}_extra_data")
+            self.extra_data.serialise(f"{context}_extra_data", bba)
 
     def serialise(self, context: str, bba: BBAWriter):
         bba.u32(self.name.index)
         bba.slice(f"{context}_pads", len(self.pads))
+        if self.extra_data is not None:
+            bba.ref(f"{context}_extra_data")
+        else:
+            bba.u32(0)
 
 class TimingValue(BBAStruct):
     def __init__(self, fast_min=0, fast_max=None, slow_min=None, slow_max=None):
@@ -580,6 +665,7 @@ class SpeedGrade(BBAStruct):
     pip_classes: list[Optional[PipTiming]] = field(default_factory=list)
     node_classes: list[Optional[NodeTiming]] = field(default_factory=list)
     cell_types: list[CellTiming] = field(default_factory=list) # sorted by (cell_type, variant) ID tuple
+    extra_data: object = None
 
     def finalise(self):
         self.cell_types.sort(key=lambda ty: ty.type_variant)
@@ -598,11 +684,19 @@ class SpeedGrade(BBAStruct):
         bba.label(f"{context}_cell_types")
         for i, t in enumerate(self.cell_types):
             t.serialise(f"{context}_cellty{i}", bba)
+        if self.extra_data is not None:
+            self.extra_data.serialise_lists(f"{context}_extra_data", bba)
+            bba.label(f"{context}_extra_data")
+            self.extra_data.serialise(f"{context}_extra_data", bba)
     def serialise(self, context: str, bba: BBAWriter):
         bba.u32(self.name.index) # speed grade idstring
         bba.slice(f"{context}_pip_classes", len(self.pip_classes))
         bba.slice(f"{context}_node_classes", len(self.node_classes))
         bba.slice(f"{context}_cell_types", len(self.cell_types))
+        if self.extra_data is not None:
+            bba.ref(f"{context}_extra_data")
+        else:
+            bba.u32(0)
 
 class TimingPool(BBAStruct):
     def __init__(self, strs: StringPool):
@@ -654,7 +748,7 @@ class TimingPool(BBAStruct):
     def set_bel_pin_class(self, grade: str, name: str, delay: TimingValue,
             in_cap: Optional[TimingValue]=None, out_res: Optional[TimingValue]=None):
         # bel pin classes are shared with pip classes, but this alias adds a bit of extra clarity
-        set_pip_class(self, grade, name, delay, in_cap, out_res, is_buffered=True)
+        self.set_pip_class(grade, name, delay, in_cap, out_res, is_buffered=True)
 
     def set_node_class(self, grade: str, name: str,  delay: TimingValue,
             res: Optional[TimingValue]=None, cap: Optional[TimingValue]=None):
@@ -670,6 +764,9 @@ class TimingPool(BBAStruct):
         self.speed_grades[self.speed_grade_idx[speed_grade]].cell_types.append(cell)
         return cell
 
+    def get_speed_grade(self, grade: str):
+        return self.speed_grades[self.speed_grade_idx[grade]]
+
     def finalise(self):
         for sg in self.speed_grades:
             sg.finalise()
@@ -691,8 +788,9 @@ class Chip:
         self.packages = []
         self.extra_data = None
         self.timing = TimingPool(self.strs)
+        self.gfx_wire_ids = dict()
     def create_tile_type(self, name: str):
-        tt = TileType(self.strs, self.timing, self.strs.id(name))
+        tt = TileType(self.strs, self.gfx_wire_ids, self.timing, self.strs.id(name))
         self.tile_type_idx[name] = len(self.tile_types)
         self.tile_types.append(tt)
         return tt
@@ -822,7 +920,7 @@ class Chip:
 
         bba.label("chip_info")
         bba.u32(0x00ca7ca7) # magic
-        bba.u32(3) # version
+        bba.u32(6) # version
         bba.u32(self.width)
         bba.u32(self.height)
 
@@ -857,3 +955,16 @@ class Chip:
             bba.ref('chip_info')
             self.serialise(bba)
             bba.pop()
+
+    def read_gfxids(self, filename):
+        idx = 1
+        with open(filename) as f:
+            for line in f:
+                l = line.strip()
+                if not l.startswith("X("):
+                    continue
+                l = l[2:]
+                assert l.endswith(")"), l
+                l = l[:-1].strip()
+                self.gfx_wire_ids[l] = idx
+                idx += 1
