@@ -1520,7 +1520,7 @@ void pack_plls(Context *ctx)
                 log_info("    Input frequency of PLL '%s' is constrained to %.1f MHz\n", ctx->nameOf(ci),
                          MHz(ctx, input_constr));
                 // Input divider (DIVR)
-                input_constr *= (int_or_default(packed->params, id_DIVR, 0) + 1);
+                auto input_divr = input_constr * (int_or_default(packed->params, id_DIVR, 0) + 1);
                 delay_t vco_constr = 0;
                 delay_t outa_constr = 0, outb_constr = 0;
                 int sr_div = 4;
@@ -1561,7 +1561,7 @@ void pack_plls(Context *ctx)
                 }
                 }
                 // Determine dividers in VCO path
-                vco_constr = input_constr / (int_or_default(packed->params, id_DIVF, 0) + 1);
+                vco_constr = input_divr / (int_or_default(packed->params, id_DIVF, 0) + 1);
                 divq = 1 << (int_or_default(packed->params, id_DIVQ, 0));
                 if (fbp_value != "1") // anything other than SIMPLE - feedback after DIVQ
                     vco_constr /= divq;
@@ -1619,6 +1619,7 @@ void pack_plls(Context *ctx)
                 bool found_lut = false;
                 bool all_luts = true;
                 bool found_carry = false;
+                bool found_ff = false;
                 unsigned int lut_count = 0;
                 for (const auto &user : port.net->users) {
                     NPNR_ASSERT(user.cell != nullptr);
@@ -1626,6 +1627,10 @@ void pack_plls(Context *ctx)
                         if (bool_or_default(user.cell->params, id_CARRY_ENABLE, false)) {
                             found_carry = true;
                             all_luts = false;
+                        } else if (bool_or_default(user.cell->params, id_DFF_ENABLE, false)) {
+                            found_lut = true;
+                            found_ff = true;
+                            lut_count++;
                         } else {
                             found_lut = true;
                             lut_count++;
@@ -1635,9 +1640,9 @@ void pack_plls(Context *ctx)
                     }
                 }
 
-                if (found_lut && all_luts && lut_count < 8) {
+                if (found_lut && all_luts && (!found_ff || lut_count == 1) && lut_count < 8) {
                     // Every user is a LUT, carry on now.
-                } else if (found_lut && !all_luts && !found_carry && lut_count < 8) {
+                } else if (found_lut && !all_luts && !found_carry && !found_ff && lut_count < 8) {
                     // Strategy: create a pass-through LUT, move all non-LUT users behind it.
                     log_info("  LUT strategy for %s: move non-LUT users to new LUT\n", port.name.c_str(ctx));
                     auto pt = spliceLUT(ctx, packed.get(), port.name, true);
