@@ -52,7 +52,7 @@ void config_empty_lcmxo3_9400(ChipConfig &cc);
 namespace {
 struct MachXO2Bitgen
 {
-    explicit MachXO2Bitgen(Context *ctx) : ctx(ctx){};
+    explicit MachXO2Bitgen(Context *ctx) : ctx(ctx) {};
     Context *ctx;
     ChipConfig cc;
     // Convert an absolute wire name to a relative Trellis one
@@ -211,11 +211,11 @@ struct MachXO2Bitgen
     {
         static const std::set<std::string> pio_t = {"PIC_T0", "PIC_T0_256", "PIC_TS0"};
         static const std::set<std::string> pio_b = {"PIC_B0", "PIC_B0_256", "PIC_BS0_256"};
-        static const std::set<std::string> pio_l = {"PIC_L0",       "PIC_L1",        "PIC_L2",        "PIC_L3",
-                                                    "PIC_LS0",      "PIC_L0_VREF3",  "PIC_L0_VREF4",  "PIC_L0_VREF5",
-                                                    "PIC_L1_VREF3", "PIC_L1_VREF4",  "PIC_L1_VREF5",  "PIC_L2_VREF4",
-                                                    "PIC_L2_VREF5", "PIC_L3_VREF4",  "PIC_L3_VREF5",  "LLC0PIC",
-                                                    "LLC1PIC",      "LLC0PIC_VREF3", "LLC3PIC_VREF3", "ULC3PIC"};
+        static const std::set<std::string> pio_l = {
+                "PIC_L0",        "PIC_L1",        "PIC_L2",       "PIC_L3",       "PIC_LS0",           "PIC_L0_VREF3",
+                "PIC_L0_VREF4",  "PIC_L0_VREF5",  "PIC_L1_VREF3", "PIC_L1_VREF4", "PIC_L1_VREF5",      "PIC_L2_VREF4",
+                "PIC_L2_VREF5",  "PIC_L3_VREF4",  "PIC_L3_VREF5", "LLC0PIC",      "LLC0PIC_I3C_VREF3", "LLC1PIC",
+                "LLC0PIC_VREF3", "LLC3PIC_VREF3", "ULC3PIC",      "PIC_L0_I3C"};
         static const std::set<std::string> pio_r = {"PIC_R0",      "PIC_R1",   "PIC_RS0",  "PIC_R0_256", "PIC_R1_640",
                                                     "PIC_RS0_256", "LRC1PIC1", "LRC1PIC2", "URC1PIC"};
 
@@ -267,7 +267,7 @@ struct MachXO2Bitgen
         if (name == "LPLL") {
             tiles.push_back(ctx->get_tile_by_type_loc(loc.y - 1, loc.x - 1, "GPLL_L0"));
         } else if (name == "RPLL") {
-            tiles.push_back(ctx->get_tile_by_type_loc(loc.y + 1, loc.x - 1, "GPLL_R0"));
+            tiles.push_back(ctx->get_tile_by_type_loc(loc.y - 1, loc.x + 1, "GPLL_R0"));
         } else {
             NPNR_ASSERT_FALSE_STR("bad PLL loc " + name);
         }
